@@ -37,34 +37,77 @@ def _parse_vaccinations_timestamp(data):
 def _parse_vaccinations_by_day(data) -> "list[VaccinationByDayRow]":
     _validate_response_data(data)
     resp = data["results"][0]["result"]["data"]["dsr"]["DS"][0]["PH"][0]["DM0"]
-    parsed_data = []
+    parsed_data: "list[VaccinationByDayRow]" = []
+
+    r_list = [None, 2, 4, 6, 8, 10, 12, 14]
 
     date = None
     people_vaccinated = None
     people_fully_vaccinated = None
+    people_third_dose = None
     for element in resp:
+
         C = element["C"]
-        if len(C) == 3:
-            date = parse_date(C[0])
+        R = element.get("R", None)
+        date = parse_date(C[0])
+
+        if R not in r_list:
+            raise Exception(f"Unknown R: {R}, date: {date}, C: {C}!")
+
+        if R == None:
             people_vaccinated = C[1]
             people_fully_vaccinated = C[2]
-        elif len(C) == 2:
-            date = parse_date(C[0])
-            R = element["R"]
-            if R == 2:
-                people_fully_vaccinated = C[1]
-            else:
-                people_vaccinated = C[1]
-        elif len(C) == 1:
-            date = parse_date(C[0])
-        else:
-            raise Exception("Unknown item length!")
+            people_third_dose = C[3]
+
+        # reuse from previous iteration
+        if R == 2:
+            # reuse first dose
+            people_vaccinated = parsed_data[-1].first_dose
+            people_fully_vaccinated = C[1]
+            people_third_dose = C[2]
+
+        if R == 4:
+            # reuse second dose
+            people_vaccinated = C[1]
+            people_fully_vaccinated = parsed_data[-1].second_dose
+            people_third_dose = C[2]
+
+        if R == 6:
+            # reuse first and second dose
+            people_vaccinated = parsed_data[-1].first_dose
+            people_fully_vaccinated = parsed_data[-1].second_dose
+            people_third_dose = C[1]
+
+        if R == 8:
+            # reuse third dose
+            people_vaccinated = C[1]
+            people_fully_vaccinated = C[2]
+            people_third_dose = parsed_data[-1].third_dose
+
+        if R == 10:
+            # reuse first and third dose
+            people_vaccinated = parsed_data[-1].first_dose
+            people_fully_vaccinated = C[1]
+            people_third_dose = parsed_data[-1].third_dose
+
+        if R == 12:
+            # reuse second and third dose
+            people_vaccinated = C[1]
+            people_fully_vaccinated = parsed_data[-1].second_dose
+            people_third_dose = parsed_data[-1].third_dose
+
+        if R == 14:
+            # reuse all doses
+            people_vaccinated = parsed_data[-1].first_dose
+            people_fully_vaccinated = parsed_data[-1].second_dose
+            people_third_dose = parsed_data[-1].third_dose
 
         parsed_data.append(
             VaccinationByDayRow(
                 date=date,
                 first_dose=people_vaccinated,
                 second_dose=people_fully_vaccinated,
+                third_dose=people_third_dose,
             )
         )
 
@@ -177,7 +220,7 @@ def _parse_vaccines_supplied_by_manufacturer(
             raise Exception("Missing manufacturer!")
         return manu_keys[num]
 
-    r_list = [None, 1, 2, 4, 6]
+    r_list = [None, 1, 2, 4, 5, 6]
 
     date = None
     manufacturer = None
@@ -221,7 +264,16 @@ def _parse_vaccines_supplied_by_manufacturer(
             date = parse_date(C[0])
             manufacturer = get_manufacturer((C[1]))
             setattr(manu_row, "date", date)
-            setattr(manu_row, manufacturer, value) # reuse value from previous iteration
+            setattr(
+                manu_row, manufacturer, value
+            )  # reuse value from previous iteration
+
+        if R == 5:
+            # same value, same date, but different manufacturer
+            manufacturer = get_manufacturer((C[0]))
+            setattr(
+                parsed_data[-1], manufacturer, value
+            )  # reuse value from previous iteration
 
         if R == 6:
             # same manufacturer and value as previous
@@ -229,7 +281,7 @@ def _parse_vaccines_supplied_by_manufacturer(
             setattr(manu_row, "date", date)
             setattr(manu_row, manufacturer, value)
 
-        if R != 1:
+        if R != 1 and R != 5:
             parsed_data.append(manu_row)
     return parsed_data
 
@@ -546,55 +598,46 @@ def _create_vaccinations_by_manufacturer_parser(manufacturer: Manufacturer):
     }
 
     def _parse_vaccinations_by_manufacturer_used(data) -> "list[VaccinationDose]":
+        # Here is possible to get data for, first, second, third and total.
+        # We need total at the moment.
         _validate_response_data(data)
         resp = data["results"][0]["result"]["data"]["dsr"]["DS"][0]["PH"][0]["DM0"]
 
-        delivery_date = Manufacturer_First_Delivery_Date[manufacturer]
-        first_date = parse_date(resp[0]["G0"])
-
-        # Someone at NIJZ (or whoever is entering data) mostlikely made a mistake.
-        # Moderna was first used on 2021-01-08 while first delivery was on 2021-01-12
-        # Astra Zeneca was first used on 2021-01-28 while first delivery was on 2021-02-06
-        is_used_before_delivered = delivery_date + DAY_DELTA > first_date
-        if is_used_before_delivered:
-            print(
-                f"{manufacturer.value} was used before it was delivered!\nFirst delivery: {delivery_date}.\nFirst use: {first_date}"
-            )
-
-        parsed_data = []
-        day_delta = datetime.timedelta(days=1)
-        previous_date = (
-            first_date if is_used_before_delivered else delivery_date + DAY_DELTA
-        )
-        used = None
+        parsed_data: "list[VaccinationDose]" = []
         for element in resp:
-            date = parse_date(element["G0"])
-            if previous_date + day_delta < date:
-                print("Populate with dates with no usage!")
-                print(f"Last data date: {previous_date}")
-                print(f"Current data date: {date}")
+            C = element.get("C")
+            R = element.get(
+                "R", None
+            )  # maybe for later if we decide to parse for each dose
+            Ø = element.get(
+                "Ø", None
+            )  # maybe for later if we decide to parse for each dose
 
-            possible_missing_date = previous_date + day_delta
+            date = parse_date(C[0])
+            total_used = C[-1]
 
-            while possible_missing_date < date:
-                # populate with dates in between
-                print(f"Add data for missing date: {possible_missing_date}")
-                parsed_data.append(VaccinationDose(possible_missing_date, None))
-                possible_missing_date += day_delta
+            # lines below might be needed if we will have to check another R
+            # suppported_r = [30, 28, 18]  # there are many Rs that we don't support.
+            # if R not in suppported_r:
+            #     print(element)
 
-            previous_date = date
+            if R == 30:
+                total_used = parsed_data[-1].dose
 
-            X = element["X"]
-            M0 = X[0].get("M0", None)
-            R = X[0].get("R", None)
-            if M0 != None:
-                parsed_data.append(VaccinationDose(date, M0))
-                used = M0
-            elif R == 1:
-                parsed_data.append(VaccinationDose(date, used))
-            else:
-                print(R, element)
-                raise Exception(f"Unknown R: {R}")
+            # I have no idea what Ø is. I can speculate that is related to doses: first, second or third
+            if R == 28 and Ø == None:
+                total_used = parsed_data[-1].dose
+
+            if R == 28 and Ø == 2:
+                total_used = parsed_data[-1].dose
+
+            if (
+                R == 18 and Ø == 12
+            ):  # ? Ø might be days between use doses AZ 2021-11-19, 2021-11-23
+                print(parsed_data[-1])
+                total_used = parsed_data[-1].dose
+
+            parsed_data.append(VaccinationDose(date, total_used))
 
         return parsed_data
 
